@@ -12,6 +12,7 @@
 
 import copy
 
+from keystoneauth1 import exceptions as ksa_exceptions
 from keystoneauth1 import plugin as ksa_plugin
 from keystoneauth1 import session as ksa_session
 import mock
@@ -157,6 +158,11 @@ class TestCloudConfig(base.TestCase):
         cc.config['volume_api_version'] = '2'
         self.assertEqual('volumev2', cc.get_service_type('volume'))
 
+    def test_volume_override_v3(self):
+        cc = cloud_config.CloudConfig("test1", "region-al", fake_services_dict)
+        cc.config['volume_api_version'] = '3'
+        self.assertEqual('volumev3', cc.get_service_type('volume'))
+
     def test_get_session_no_auth(self):
         config_dict = defaults.get_defaults()
         config_dict.update(fake_services_dict)
@@ -235,30 +241,99 @@ class TestCloudConfig(base.TestCase):
             region_name='region-al',
             service_type='orchestration')
 
+    @mock.patch.object(cloud_config.CloudConfig, 'get_session')
+    def test_session_endpoint_not_found(self, mock_get_session):
+        exc_to_raise = ksa_exceptions.catalog.EndpointNotFound
+        mock_get_session.return_value.get_endpoint.side_effect = exc_to_raise
+        cc = cloud_config.CloudConfig(
+            "test1", "region-al", {}, auth_plugin=mock.Mock())
+        self.assertIsNone(cc.get_session_endpoint('notfound'))
+
+    @mock.patch.object(cloud_config.CloudConfig, 'get_api_version')
+    @mock.patch.object(cloud_config.CloudConfig, 'get_auth_args')
     @mock.patch.object(cloud_config.CloudConfig, 'get_session_endpoint')
-    def test_legacy_client_object_store(self, mock_get_session_endpoint):
+    def test_legacy_client_object_store_password(
+            self,
+            mock_get_session_endpoint,
+            mock_get_auth_args,
+            mock_get_api_version):
         mock_client = mock.Mock()
-        mock_get_session_endpoint.return_value = 'http://example.com/v2'
+        mock_get_session_endpoint.return_value = 'http://swift.example.com'
+        mock_get_api_version.return_value = '3'
+        mock_get_auth_args.return_value = dict(
+            username='testuser',
+            password='testpassword',
+            project_name='testproject',
+            auth_url='http://example.com',
+        )
         config_dict = defaults.get_defaults()
         config_dict.update(fake_services_dict)
         cc = cloud_config.CloudConfig(
             "test1", "region-al", config_dict, auth_plugin=mock.Mock())
         cc.get_legacy_client('object-store', mock_client)
         mock_client.assert_called_with(
-            preauthtoken=mock.ANY,
+            session=mock.ANY,
             os_options={
-                'auth_token': mock.ANY,
                 'region_name': 'region-al',
-                'object_storage_url': 'http://example.com/v2'
-            },
-            preauthurl='http://example.com/v2',
-            auth_version='2.0')
+                'service_type': 'object-store',
+                'object_storage_url': None,
+                'endpoint_type': 'public',
+            })
 
+    @mock.patch.object(cloud_config.CloudConfig, 'get_auth_args')
     @mock.patch.object(cloud_config.CloudConfig, 'get_session_endpoint')
-    def test_legacy_client_object_store_timeout(
-            self, mock_get_session_endpoint):
+    def test_legacy_client_object_store_password_v2(
+            self, mock_get_session_endpoint, mock_get_auth_args):
+        mock_client = mock.Mock()
+        mock_get_session_endpoint.return_value = 'http://swift.example.com'
+        mock_get_auth_args.return_value = dict(
+            username='testuser',
+            password='testpassword',
+            project_name='testproject',
+            auth_url='http://example.com',
+        )
+        config_dict = defaults.get_defaults()
+        config_dict.update(fake_services_dict)
+        cc = cloud_config.CloudConfig(
+            "test1", "region-al", config_dict, auth_plugin=mock.Mock())
+        cc.get_legacy_client('object-store', mock_client)
+        mock_client.assert_called_with(
+            session=mock.ANY,
+            os_options={
+                'region_name': 'region-al',
+                'service_type': 'object-store',
+                'object_storage_url': None,
+                'endpoint_type': 'public',
+            })
+
+    @mock.patch.object(cloud_config.CloudConfig, 'get_auth_args')
+    @mock.patch.object(cloud_config.CloudConfig, 'get_session_endpoint')
+    def test_legacy_client_object_store(
+            self, mock_get_session_endpoint, mock_get_auth_args):
         mock_client = mock.Mock()
         mock_get_session_endpoint.return_value = 'http://example.com/v2'
+        mock_get_auth_args.return_value = {}
+        config_dict = defaults.get_defaults()
+        config_dict.update(fake_services_dict)
+        cc = cloud_config.CloudConfig(
+            "test1", "region-al", config_dict, auth_plugin=mock.Mock())
+        cc.get_legacy_client('object-store', mock_client)
+        mock_client.assert_called_with(
+            session=mock.ANY,
+            os_options={
+                'region_name': 'region-al',
+                'service_type': 'object-store',
+                'object_storage_url': None,
+                'endpoint_type': 'public',
+            })
+
+    @mock.patch.object(cloud_config.CloudConfig, 'get_auth_args')
+    @mock.patch.object(cloud_config.CloudConfig, 'get_session_endpoint')
+    def test_legacy_client_object_store_timeout(
+            self, mock_get_session_endpoint, mock_get_auth_args):
+        mock_client = mock.Mock()
+        mock_get_session_endpoint.return_value = 'http://example.com/v2'
+        mock_get_auth_args.return_value = {}
         config_dict = defaults.get_defaults()
         config_dict.update(fake_services_dict)
         config_dict['api_timeout'] = 9
@@ -266,33 +341,33 @@ class TestCloudConfig(base.TestCase):
             "test1", "region-al", config_dict, auth_plugin=mock.Mock())
         cc.get_legacy_client('object-store', mock_client)
         mock_client.assert_called_with(
-            preauthtoken=mock.ANY,
+            session=mock.ANY,
             os_options={
-                'auth_token': mock.ANY,
                 'region_name': 'region-al',
-                'object_storage_url': 'http://example.com/v2'
-            },
-            preauthurl='http://example.com/v2',
-            auth_version='2.0',
-            timeout=9.0)
+                'service_type': 'object-store',
+                'object_storage_url': None,
+                'endpoint_type': 'public',
+            })
 
-    def test_legacy_client_object_store_endpoint(self):
+    @mock.patch.object(cloud_config.CloudConfig, 'get_auth_args')
+    def test_legacy_client_object_store_endpoint(
+            self, mock_get_auth_args):
         mock_client = mock.Mock()
+        mock_get_auth_args.return_value = {}
         config_dict = defaults.get_defaults()
         config_dict.update(fake_services_dict)
-        config_dict['object_store_endpoint'] = 'http://example.com/v2'
+        config_dict['object_store_endpoint'] = 'http://example.com/swift'
         cc = cloud_config.CloudConfig(
             "test1", "region-al", config_dict, auth_plugin=mock.Mock())
         cc.get_legacy_client('object-store', mock_client)
         mock_client.assert_called_with(
-            preauthtoken=mock.ANY,
+            session=mock.ANY,
             os_options={
-                'auth_token': mock.ANY,
                 'region_name': 'region-al',
-                'object_storage_url': 'http://example.com/v2'
-            },
-            preauthurl='http://example.com/v2',
-            auth_version='2.0')
+                'service_type': 'object-store',
+                'object_storage_url': 'http://example.com/swift',
+                'endpoint_type': 'public',
+            })
 
     @mock.patch.object(cloud_config.CloudConfig, 'get_session_endpoint')
     def test_legacy_client_image(self, mock_get_session_endpoint):
@@ -449,7 +524,7 @@ class TestCloudConfig(base.TestCase):
         mock_client.assert_called_with(
             version='2.0',
             endpoint='http://example.com/v2',
-            endpoint_type='admin',
+            interface='admin',
             endpoint_override=None,
             region_name='region-al',
             service_type='identity',
@@ -469,7 +544,7 @@ class TestCloudConfig(base.TestCase):
         mock_client.assert_called_with(
             version='3',
             endpoint='http://example.com',
-            endpoint_type='admin',
+            interface='admin',
             endpoint_override=None,
             region_name='region-al',
             service_type='identity',
